==> jekyll-embed-urls-0.4.0.gem: lib/jekyll/embed.rb <==
# frozen_string_literal: true

require 'faraday'
require 'faraday-http-cache'
require 'faraday_middleware/response/follow_redirects'
require 'loofah'
require 'ogp'
require 'url_privacy'

require_relative 'embed/oembed'
require_relative 'embed/cache'

if Gem::Version.new(Jekyll::VERSION) >= Gem::Version.new('4.2.0')
  require_relative 'embed/content'
else
  Jekyll.logger.warn "Upgrade to Jekyll >= 4.2.0 to embed URLs in content"
end

OEmbed::Providers.register_all
OEmbed::Providers.register_fallback(OEmbed::ProviderDiscovery,
                                    OEmbed::Providers::Noembed)

module Jekyll
  # The idea with this class is to find the best safe representation of
  # a link.  For a YouTube video it could be the sandboxed iframe.  This
  # loads the video and allows you to reproduce it while preventing YT
  # to call home and send data about your users.  But other social networks
  # will try to take control of their containers by modifying the page.
  # They resist sandboxing and don't work correctly.  For them, we
  # cleanup unwanted HTML tags such as <script>, and return the HTML,
  # which you can style using CSS.  Twitter does this.
  #
  # Others are only available through OGP, so we retrieve the metadata
  # and render a template, which you can provide in your own theme too.
  #
  # We also try for microformats and we would look at Schema.org too but
  # doesn't seem to be a gem for it yet.
  #
  # If the URL doesn't provide anything at all we get the URL, title and
  # date of last visit.
  #
  # Isn't it nice that the corporations that requires us to use OEmbed,
  # OGP, Twitter Cards, Schema.org and other metadata, don't do use
  # themselves?
  #
  # Also we're going to use heavy caching so we don't hit rate limits or
  # lose the representation if the service is down or the URL is
  # removed.  We may be tempted to store the resources locally (images,
  # videos, audio) but we have to take into account that people have
  # legitimate reasons to remove media from the Internet.
  class Embed
    # Attributes to apply by HTMLElement
    IFRAME_ATTRIBUTES = %w[allow sandbox referrerpolicy loading].freeze
    IMAGE_ATTRIBUTES = %w[referrerpolicy loading].freeze
    MEDIA_ATTRIBUTES = %w[controls].freeze
    A_ATTRIBUTES = %w[referrerpolicy rel target].freeze

    # Templates
    INCLUDE_OGP = '{% include ogp.html site=site page=page %}'
    INCLUDE_FALLBACK = '{% include fallback.html site=site page=page %}'

    # The default referrer policy only sends the origin URL (not the
    # full URL, only the protocol/scheme and domain part) if the remote
    # URL is HTTPS.
    #
    # @see {https://developer.mozilla.org/en-US/docs/Web/HTTP/Headers/Referrer-Policy}
    #
    # The default sandbox restrictions only allow scripts in the context
    # of the iframe and opening new tabs.
    #
    # @see {https://developer.mozilla.org/en-US/docs/Web/HTML/Element/iframe#attr-sandbox}
    DEFAULT_CONFIG = {
      'scrub' => %w[form input textarea button fieldset select option optgroup canvas area map],
      'attributes' => {
        'referrerpolicy' => 'strict-origin-when-cross-origin',
        'sandbox' => %w[allow-scripts allow-popups],
        'allow' => %w[fullscreen gyroscope picture-in-picture clipboard-write],
        'loading' => 'lazy',
        'controls' => true,
        'rel' => %w[noopener noreferrer],
        'target' => '_blank'
      }
    }

    class << self
      def site
        unless @site
          raise Jekyll::Errors::InvalidConfigurationError,
            "Site is missing, configure with `Jekyll::Embed.site = site`"
        end

        @site
      end

      # This is an initializer of sorts
      #
      # @param [Jekyll::Site]
      # @return [Jekyll::Site]
      def site=(site)
        raise ArgumentError, "Site must be a Jekyll::Site" unless site.is_a? Jekyll::Site

        @site = site

        # Add the _includes dir so we can provide default templates that
        # can be overriden locally or by the theme.
        site.includes_load_paths << File.expand_path(File.join(__dir__, '..', '..', '_includes'))
        # Since we're embedding, we're allowing iframes
        Loofah::HTML5::SafeList::ALLOWED_ELEMENTS_WITH_LIBXML2 << 'iframe'

        # Other elements that are disallowed
        config['scrub']&.each do |scrub|
          Loofah::HTML5::SafeList::ALLOWED_ELEMENTS_WITH_LIBXML2.delete(scrub)
        end

        payload['embed'] = config['attributes']

        site
      end

      # Render the URL as HTML
      #
      # 1. Try oembed for video and image
      # 2. If rich oembed, cleanup
      # 3. If OGP, render templates
      # 4. Else, render fallback template
      #
      # @param [String] URL
      # @return [String] HTML
      def embed(url)
        url.strip!

        # Quick check
        raise URI::Error unless url.start_with? 'http'

        # Just to verify the URL is valid
        URI.parse url

        oembed(url) || ogp(url) || fallback(url)
      rescue URI::Error
        Jekyll.logger.warn "#{url.inspect} is not a valid URL"

        url
      end

      # @return [Hash]
      def config
        @config ||= Jekyll::Utils.deep_merge_hashes(DEFAULT_CONFIG, (site.config['embed'] || {}))
      end

      # Try for OEmbed
      #
      # @param [String] URL
      # @return [String,NilClass] Sanitized HTML or nil
      def oembed(url)
        cache.getset(url) do
          oembed = OEmbed::Providers.get url

          # Prevent caching of nil?
          raise OEmbed::Error unless oembed.respond_to? :html

          # Cleanup.  We don't allow running remote scripts locally,
          # period.
          cleanup(Loofah.fragment(oembed.html).scrub!(:prune), url).to_s
        end
      rescue OEmbed::Error
        nil
      end

      # Try for OGP.
      # @param [String] URL
      # @return [String,NilClass]
      def ogp(url)
        cache.getset(url) do
          ogp = OGP::OpenGraph.new get(url).body
          context = info.dup
          context[:registers][:page] = payload['page'] = ogp.data

          ogp_template.render! payload, context
        end
      rescue OGP::MalformedSourceError, OGP::MissingAttributeError, Faraday::Error
        nil
      end

      # Try something
      def fallback(url)
        cache.getset(url) do
          html        = Nokogiri::HTML.fragment get(url).body
          element     = html.css('article').first
          element   ||= html.css('section').first
          element   ||= html.css('main').first
          element   ||= html.css('body').first
          title       = html.css('title').first
          description = html.css('meta[name="description"]').first

          context = info.dup
          context[:registers][:page] = payload['page'] = {
            'title' => text(title),
            'description' => text(description),
            'url' => url,
            'image' => element.css('img').first&.public_send(:[], 'src')
          }

          fallback_template.render! payload, context
        end
      rescue Faraday::Error, Nokogiri::SyntaxError
        nil
      end

      # @param [String] URL
      # @return [Faraday::Response]
      def get(url)
        @get_cache ||= {}
        @get_cache[url] ||= http_client.get url
      end

      # @return [Jekyll::Embed::Cache]
      def cache
        @cache ||= Jekyll::Embed::Cache.new('Jekyll::Embed')
      end

      # @return [Faraday::Connection]
      def http_client
        @http_client ||= Faraday.new do |builder|
          builder.use FaradayMiddleware::FollowRedirects
          builder.use :http_cache, shared_cache: false, store: cache, serializer: Marshal
        end
      end

      def cleanup(html, url)
        # Add our own attributes
        html.css('iframe').each do |iframe|
          IFRAME_ATTRIBUTES.each do |attr|
            iframe[attr] = value_for_attr(attr)
          end

          # Embedding itself require allow-same-origin
          iframe['sandbox'] += allow_same_origin(url)
        end

        html.css('audio, video').each do |media|
          MEDIA_ATTRIBUTES.each do |attr|
            media[attr] = value_for_attr(attr)
          end

          media['src'] = UrlPrivacy.clean media['src']
        end

        html.css('img').each do |img|
          IMAGE_ATTRIBUTES.each do |attr|
            img[attr] = value_for_attr(attr)
          end
        end

        html.css('a').each do |a|
          A_ATTRIBUTES.each do |attr|
            a[attr] = value_for_attr(attr)
          end
        end

        html.css('[src]').each do |element|
          element['src'] = CGI.escapeHTML(UrlPrivacy.clean(CGI.unescapeHTML(element['src'])))
        end

        html.css('[href]').each do |element|
          element['href'] = CGI.escapeHTML(UrlPrivacy.clean(CGI.unescapeHTML(element['href'])))
        end

        # Return the cleaned up HTML
        html
      end

      def text(node)
        node&.text&.tr("\n", '')&.tr("\r", '')&.strip&.squeeze(' ')
      end

      private

      def fallback_template
        @fallback_template ||= site.liquid_renderer.file('fallback.html').parse(INCLUDE_FALLBACK)
      end

      def ogp_template
        @ogp_template ||= site.liquid_renderer.file('ogp.html').parse(INCLUDE_OGP)
      end

      def info
        @info ||= {
          registers: { site: site },
          strict_filters: site.config.dig('liquid', 'strict_filters'),
          strict_variables: site.config.dig('liquid', 'strict_variables')
        }
      end

      # @param [String]
      # @return [String]
      def value_for_attr(attr)
        @value_for_attr ||= {}
        @value_for_attr[attr] ||=
          case (value = config.dig('attributes', attr))
            when String then value
            when Array then value.join(' ')
          end
      end

      # If the iframe comes from the same site, we can allow the same
      # origin policy on the sandbox.
      #
      # @param [String] URL
      # @return [String]
      def allow_same_origin(url)
        unless site.config['url']
          Jekyll.logger.warn "Add url to _config.yml to determine if the site can embed itself"
          return ' allow-same-origin'
        end

        @allow_same_origin ||= {}
        @allow_same_origin[url] ||= url.start_with?(site.config['url']) ? '' : ' allow-same-origin'
      end

      # Caches it because Jekyll::Site#site_payload returns a new object
      # everytime.
      #
      # @return [Jekyll::Drops::UnifiedPayloadDrop]
      def payload
        @payload ||= site.site_payload
      end
    end
  end
end

==> jekyll-embed-urls-0.4.1.gem: lib/jekyll/embed.rb <==
# frozen_string_literal: true

require 'faraday'
require 'faraday-http-cache'
require 'faraday_middleware/response/follow_redirects'
require 'loofah'
require 'ogp'
require 'url_privacy'

require_relative 'embed/oembed'
require_relative 'embed/cache'

if Gem::Version.new(Jekyll::VERSION) >= Gem::Version.new('4.2.0')
  require_relative 'embed/content'
else
  Jekyll.logger.warn "Upgrade to Jekyll >= 4.2.0 to embed URLs in content"
end

OEmbed::Providers.register_all
OEmbed::Providers.register_fallback(OEmbed::ProviderDiscovery,
                                    OEmbed::Providers::Noembed)

module Jekyll
  # The idea with this class is to find the best safe representation of
  # a link.  For a YouTube video it could be the sandboxed iframe.  This
  # loads the video and allows you to reproduce it while preventing YT
  # to call home and send data about your users.  But other social networks
  # will try to take control of their containers by modifying the page.
  # They resist sandboxing and don't work correctly.  For them, we
  # cleanup unwanted HTML tags such as <script>, and return the HTML,
  # which you can style using CSS.  Twitter does this.
  #
  # Others are only available through OGP, so we retrieve the metadata
  # and render a template, which you can provide in your own theme too.
  #
  # We also try for microformats and we would look at Schema.org too but
  # doesn't seem to be a gem for it yet.
  #
  # If the URL doesn't provide anything at all we get the URL, title and
  # date of last visit.
  #
  # Isn't it nice that the corporations that requires us to use OEmbed,
  # OGP, Twitter Cards, Schema.org and other metadata, don't do use
  # themselves?
  #
  # Also we're going to use heavy caching so we don't hit rate limits or
  # lose the representation if the service is down or the URL is
  # removed.  We may be tempted to store the resources locally (images,
  # videos, audio) but we have to take into account that people have
  # legitimate reasons to remove media from the Internet.
  class Embed
    # Attributes to apply by HTMLElement
    IFRAME_ATTRIBUTES = %w[allow sandbox referrerpolicy loading].freeze
    IMAGE_ATTRIBUTES = %w[referrerpolicy loading].freeze
    MEDIA_ATTRIBUTES = %w[controls].freeze
    A_ATTRIBUTES = %w[referrerpolicy rel target].freeze

    # Templates
    INCLUDE_OGP = '{% include ogp.html site=site page=page %}'
    INCLUDE_FALLBACK = '{% include fallback.html site=site page=page %}'

    # The default referrer policy only sends the origin URL (not the
    # full URL, only the protocol/scheme and domain part) if the remote
    # URL is HTTPS.
    #
    # @see {https://developer.mozilla.org/en-US/docs/Web/HTTP/Headers/Referrer-Policy}
    #
    # The default sandbox restrictions only allow scripts in the context
    # of the iframe and opening new tabs.
    #
    # @see {https://developer.mozilla.org/en-US/docs/Web/HTML/Element/iframe#attr-sandbox}
    DEFAULT_CONFIG = {
      'scrub' => %w[form input textarea button fieldset select option optgroup canvas area map],
      'attributes' => {
        'referrerpolicy' => 'strict-origin-when-cross-origin',
        'sandbox' => %w[allow-scripts allow-popups],
        'allow' => %w[fullscreen gyroscope picture-in-picture clipboard-write],
        'loading' => 'lazy',
        'controls' => true,
        'rel' => %w[noopener noreferrer],
        'target' => '_blank'
      }
    }

    class << self
      def site
        unless @site
          raise Jekyll::Errors::InvalidConfigurationError,
            "Site is missing, configure with `Jekyll::Embed.site = site`"
        end

        @site
      end

      # This is an initializer of sorts
      #
      # @param [Jekyll::Site]
      # @return [Jekyll::Site]
      def site=(site)
        raise ArgumentError, "Site must be a Jekyll::Site" unless site.is_a? Jekyll::Site

        @site = site

        # Add the _includes dir so we can provide default templates that
        # can be overriden locally or by the theme.
        site.includes_load_paths << File.expand_path(File.join(__dir__, '..', '..', '_includes'))
        # Since we're embedding, we're allowing iframes
        Loofah::HTML5::SafeList::ALLOWED_ELEMENTS_WITH_LIBXML2 << 'iframe'

        # Other elements that are disallowed
        config['scrub']&.each do |scrub|
          Loofah::HTML5::SafeList::ALLOWED_ELEMENTS_WITH_LIBXML2.delete(scrub)
        end

        payload['embed'] = config['attributes']

        site
      end

      # Render the URL as HTML
      #
      # 1. Try oembed for video and image
      # 2. If rich oembed, cleanup
      # 3. If OGP, render templates
      # 4. Else, render fallback template
      #
      # @param [String] URL
      # @return [String] HTML
      def embed(url)
        url.strip!

        # Quick check
        raise URI::Error unless url.start_with? 'http'

        # Just to verify the URL is valid
        URI.parse url

        oembed(url) || ogp(url) || fallback(url)
      rescue URI::Error
        Jekyll.logger.warn "#{url.inspect} is not a valid URL"

        url
      end

      # @return [Hash]
      def config
        @config ||= Jekyll::Utils.deep_merge_hashes(DEFAULT_CONFIG, (site.config['embed'] || {}))
      end

      # Try for OEmbed
      #
      # @param [String] URL
      # @return [String,NilClass] Sanitized HTML or nil
      def oembed(url)
        cache.getset(url) do
          oembed = OEmbed::Providers.get url

          # Prevent caching of nil?
          raise OEmbed::Error unless oembed.respond_to? :html

          # Cleanup.  We don't allow running remote scripts locally,
          # period.
          cleanup(Loofah.fragment(oembed.html).scrub!(:prune), url).to_s
        end
      rescue OEmbed::Error
        nil
      end

      # Try for OGP.
      # @param [String] URL
      # @return [String,NilClass]
      def ogp(url)
        cache.getset(url) do
          ogp = OGP::OpenGraph.new get(url).body
          context = info.dup
          context[:registers][:page] = payload['page'] = ogp.data

          ogp_template.render! payload, context
        end
      rescue OGP::MalformedSourceError, OGP::MissingAttributeError, Faraday::Error
        nil
      end

      # Try something
      def fallback(url)
        cache.getset(url) do
          html        = Nokogiri::HTML.fragment get(url).body
          element     = html.css('article').first
          element   ||= html.css('section').first
          element   ||= html.css('main').first
          element   ||= html.css('body').first
          title       = html.css('title').first
          description = html.css('meta[name="description"]').first

          context = info.dup
          context[:registers][:page] = payload['page'] = {
            'title' => text(title),
            'description' => text(description),
            'url' => url,
            'image' => element&.css('img').first&.public_send(:[], 'src')
          }

          fallback_template.render! payload, context
        end
      rescue Faraday::Error, Nokogiri::SyntaxError
        nil
      end

      # @param [String] URL
      # @return [Faraday::Response]
      def get(url)
        @get_cache ||= {}
        @get_cache[url] ||= http_client.get url
      end

      # @return [Jekyll::Embed::Cache]
      def cache
        @cache ||= Jekyll::Embed::Cache.new('Jekyll::Embed')
      end

      # @return [Faraday::Connection]
      def http_client
        @http_client ||= Faraday.new do |builder|
          builder.use FaradayMiddleware::FollowRedirects
          builder.use :http_cache, shared_cache: false, store: cache, serializer: Marshal
        end
      end

      def cleanup(html, url)
        # Add our own attributes
        html.css('iframe').each do |iframe|
          IFRAME_ATTRIBUTES.each do |attr|
            iframe[attr] = value_for_attr(attr)
          end

          # Embedding itself require allow-same-origin
          iframe['sandbox'] += allow_same_origin(url)
        end

        html.css('audio, video').each do |media|
          MEDIA_ATTRIBUTES.each do |attr|
            media[attr] = value_for_attr(attr)
          end

          media['src'] = UrlPrivacy.clean media['src']
        end

        html.css('img').each do |img|
          IMAGE_ATTRIBUTES.each do |attr|
            img[attr] = value_for_attr(attr)
          end
        end

        html.css('a').each do |a|
          A_ATTRIBUTES.each do |attr|
            a[attr] = value_for_attr(attr)
          end
        end

        html.css('[src]').each do |element|
          element['src'] = CGI.escapeHTML(UrlPrivacy.clean(CGI.unescapeHTML(element['src'])))
        end

        html.css('[href]').each do |element|
          element['href'] = CGI.escapeHTML(UrlPrivacy.clean(CGI.unescapeHTML(element['href'])))
        end

        # Return the cleaned up HTML
        html
      end

      def text(node)
        node&.text&.tr("\n", '')&.tr("\r", '')&.strip&.squeeze(' ')
      end

      private

      def fallback_template
        @fallback_template ||= site.liquid_renderer.file('fallback.html').parse(INCLUDE_FALLBACK)
      end

      def ogp_template
        @ogp_template ||= site.liquid_renderer.file('ogp.html').parse(INCLUDE_OGP)
      end

      def info
        @info ||= {
          registers: { site: site },
          strict_filters: site.config.dig('liquid', 'strict_filters'),
          strict_variables: site.config.dig('liquid', 'strict_variables')
        }
      end

      # @param [String]
      # @return [String]
      def value_for_attr(attr)
        @value_for_attr ||= {}
        @value_for_attr[attr] ||=
          case (value = config.dig('attributes', attr))
            when String then value
            when Array then value.join(' ')
          end
      end

      # If the iframe comes from the same site, we can allow the same
      # origin policy on the sandbox.
      #
      # @param [String] URL
      # @return [String]
      def allow_same_origin(url)
        unless site.config['url']
          Jekyll.logger.warn "Add url to _config.yml to determine if the site can embed itself"
          return ' allow-same-origin'
        end

        @allow_same_origin ||= {}
        @allow_same_origin[url] ||= url.start_with?(site.config['url']) ? '' : ' allow-same-origin'
      end

      # Caches it because Jekyll::Site#site_payload returns a new object
      # everytime.
      #
      # @return [Jekyll::Drops::UnifiedPayloadDrop]
      def payload
        @payload ||= site.site_payload
      end
    end
  end
end
